@@ -728,7 +728,7 @@ def sylv_cred_rv(P_mu, P_sigma, tv_mu, tv_sigma, N_mu, N_sigma, psr,
     :param formula: formula index (1 to 7).
 
     .. Note:: Assumes that variables (P, vr, vp) are random variates (returns expected value of function, using PaCAL packages to model random variates, assuming normal distribution for all three variables).
-    Can use either PaCAL numerical integration (sssslow!), or custom numerical integration using Monte Carlo sampling (default).   
+        Can use either PaCAL numerical integration (sssslow!), or custom numerical integration using Monte Carlo sampling (default).   
     """
     if treatment_type and cover_type:
         formula = sylv_cred_formula(treatment_type, cover_type)
@@ -844,12 +844,12 @@ def harv_cost_rv(tv_mu, tv_sigma, N_mu, N_sigma, psr,
     """
     Returns harvest cost.
 
-    
     :param bool is_finalcut: Treatment type (final cut or not).
     :param bool is_toleranthw: Stand type (tolerant hardwood or not).
     :param bool partialcut_extracare: Partialcut "extra care" flag.
     :param float A: Series of regression coefficients (A, B, C, D, E, F, G, K, all with defaults that are extracted from MERIS technical documentation; also see Sebastien Lacroix, BMMB).
     :param bool rv: Types of variables (default: Variables random variates).
+
     Can use either PaCAL numerical integration (sssslow!), or custom numerical integration using Monte Carlo sampling (default).       
     """
     
@@ -903,223 +903,4 @@ def harv_cost_wec(piece_size,
     return sum(harv_cost(x, is_finalcut, is_toleranthw, **kwargs) * sigma * rv.pdf(x) for x in X)
         
 
-# class rvquot_gen(scipy.stats.rv_continuous):
-#     def __init__(self, 
-#                  locn, scalen, 
-#                  locd, scaled,
-#                  a=0.,
-#                  b=1.,
-#                  name='rvquot'):
-#         super(rvquot_gen, self).__init__(a=a,b=b,name=name)
-#         self.pacal_dist = pacal.NormalDistr(locn, scalen) / pacal.NormalDistr(locd, scaled)
-#         self.integral = scipy.integrate.quad(self.f, self.a, self.b)[0]
-
-#     def f(self, x):
-#         return self.pacal_dist.pdf(x)
-        
-#     def _pdf(self, x):
-#         return self.f(x)/self.integral
-
-
-# class rvquot_gen(scipy.stats.rv_continuous):
-#     def __init__(self, 
-#                  locn, scalen, 
-#                  locd, scaled,
-#                  a=0.0,
-#                  b=1.0,
-#                  name='rvquot',
-#                  loc=0.):
-#         super(rvquot_gen, self).__init__(a=a,b=b,name=name)
-#         self.pacal_dist = pacal.NormalDistr(locn, scalen) / pacal.NormalDistr(locd, scaled)
-#         self.loc = loc
-#         self.integral = scipy.integrate.quad(self.f, self.a, self.b)[0]
-        
-#     def f(self, x):
-#         return self.pacal_dist.pdf(x-self.loc)
-        
-#     def _pdf(self, x):
-#         return self.f(x)/self.integral
-
-
-class Node:
-    def __init__(self, nid, data=None, parent=None):
-        self.nid = nid
-        self._data = data
-        self._parent = parent
-        self._children = []
-
-    def is_root(self):
-        """
-        The function checks if the current object is the root node.
-        
-        :return: True if the object is the root node, False otherwise
-    
-        """
-        return self._parent is None
-
-    def is_leaf(self):
-        """
-        The function checks if the current object is a leaf node with no children.
-        
-        :return: True if the object is a leaf node, False otherwise
-    
-        """
-        return not self._children
-
-    def add_child(self, child):
-        """
-        The function adds a child node to the current object.
-        
-        :param child: The child node to be added.
-    
-        """
-        self._children.append(child)
-
-    def parent(self):
-        """
-        The function gets the parent node of the current object.
-       
-        :return: The parent node.
-    
-        """
-        return self._parent
-
-    def children(self):
-        """
-        The function gets the list of child nodes of the current object.
-        
-        :return: List of child nodes.
-    
-        """ 
-        return self._children
-    
-    def data(self, key=None):
-        """
-        The function gets the data associated with the current object.
-        If a specific key is provided, return the corresponding value.
-        If no key is provided, return the entire data dictionary.
-        
-        :param key: The key to retrieve a specific value (default is None).   
-        """
-        if key:
-            return self._data[key]
-        else:
-            return self._data # if not self.is_root() else None
-
-#from graphviz import Digraph        
-class Tree:
-    """
-    Represents a tree object.
-    
-    """
-    def __init__(self, period=1):
-        self._period = period
-        self._nodes = [Node(0)]
-        self._path = [self._nodes[0]]
-
-    def children(self, nid):
-        """
-        The function gets the child nodes of the node with the specified ID.
-        
-        :param nid: The ID of the node to retrieve children for.    
-        """
-        return [self._nodes[cid] for cid in self._nodes[nid].children()]
-        
-    def nodes(self):
-        """
-        The function returns all nodes
-    
-        """
-        return self._nodes
-
-    def node(self, nid):
-        """
-        The function retruns a node with the specified ID.
-        
-        :param nid: The unique identifier of the node to be retrieved.    
-        """
-        return self._nodes[nid]
-    
-    def add_node(self, data, parent=None):
-        """
-        The function adds a new node.
-        
-        :param data: The data associated with the new node.
-        :param parent: The parent node to which the new node will be attached.    
-        """
-        n = Node(len(self._nodes), data, parent)
-        self._nodes.append(n)
-        return n
-
-    def grow(self, data):
-        """
-        The function expands the current path by adding a new child node.
-        
-        :param data: The data associated with the new node.   
-        """
-        parent = self._path[-1]
-        child = self.add_node(data, parent=parent.nid)
-        parent.add_child(child.nid)
-        self._path.append(child)
-        return child
-        
-    def ungrow(self):
-        """
-        The function reduces the current path by removing the last node.
-    
-        """
-        self._path.pop()
-        
-    def leaves(self):
-        """
-        The function retruns all leaf nodes.
-    
-        """
-        return [n for n in self._nodes if n.is_leaf()]
-    
-    def root(self):
-        """
-        The function retruns the root node.
-    
-        """
-        return self._nodes[0]
-    
-    #def path(self):
-    #    return self._path
-    
-    #def period(self):
-    #    return len(self._path) - 1
-
-    def path(self, leaf=None):
-        """
-        The function retrieves the path from the root to a specific leaf node or to the currect path.
-        
-        :param leaf: The leaf node for which the path is to be retrieved (Default if None).
-        """
-        if not leaf: return self._path[1:]
-        path = []
-        n = leaf
-        while not (n.is_root()):
-            path.append(n)
-            parent = self.node(n.parent())
-            n=parent
-        path.reverse()
-        return tuple(path)
-    
-    def paths(self):
-        """
-        The function retrieves paths from the root to all leaf nodes.
-    
-        """
-        return [self.path(leaf) for leaf in self.leaves()]
-    
-    #def draw(self):
-    #    graph = Digraph()
-    #    for n in self.nodes():
-    #        print n.data()
-    #        graph.node(str(n.nid), n.data())
-    #    for n in self.nodes():
-    #        for c in self.children(n.nid):
-    #            graph.edge(str(n.nid), str(c.nid))
-    #    graph.graph_attr.update(size='10,10')
-    #    return graph
\ No newline at end of file
+ 
\ No newline at end of file
